package pinball;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

/**
 * Used to keep track of the statistics of the game.
 */
public class Stats {
	/** Number of balls played. */
	protected int balls=0;

	/** Number of times hit. */
	protected int hits=0;

	/** Total number of hits. */
	protected int hitsTotal=0;

	/** Time game started. */
	protected long startGameTime=System.currentTimeMillis();

	/** Time ball started. */
	protected long startBallTime=System.currentTimeMillis();

	/** Last time. Can be used to test for no movement. Not yest implimented.*/
	protected long lastTime=System.currentTimeMillis();	

	/** Is a ball in play. */
	protected boolean ballInPlay=false;

	/** Total time spent playing. */
	protected long totalTime=0;
	
	/** Time for that ball. */
	protected long ballTime=0;
	
	/** The pinball controlls of the system. */
	protected Stub stub;
	
	/** Time to wait from the time the ball is out of play,
	 *  before launching the ball. 
	 */
	protected final long launchTime=5000;
	
	/** The time the ball was last seen. */
	protected long lastSeen=0;

	/** The amount of time a ball can be missing until it's
	 * considered out of play. 
	 */
	protected final long missingTime=5000;
	
	/** Is the ball launchable. */
	protected boolean ballLaunchable=false;

	/** Was the ball launched. */
	protected boolean ballLaunched=false;
	
	/** Time when the ball was out of play. */
	protected long stoppedTime=0;
	
	/** When was the ball last checked for being launchable. */
	protected long lastCheckedLaunchable=0;

	/** Use to check for a start of a new game. */				
	protected StartButton startButton;
	
	
	/** 
	 * Checks the stats of the game. 
	 *
	 * @param s the the Stub used for locating the ball.
	 */
	public Stats(Stub s) {
		stub=s;
	}


	/**
	 * Get the total time the game was in use. 
	 *
	 * @return time in use.
	 */
	public long totalTime() {
		return System.currentTimeMillis()-startGameTime;
	}

	/**
	 * Ball was hit, increment the number of hits. 
	 *
	 */
	public void hitsInc() {
		hits++;
	}
	
	/**
	 * Get the number of hits.
	 *
	 * @return number of hits for this ball.
	 */
	public int hits() {
		return hits;
	}
	
	/**
	 * Increment the number of balls played so far.
	 *
	 */
	public void ballsInc() {
		balls++;
	}
	
	/**
	 * Get the number of balls played.
	 *
	 * @return number of balls played.
	 */
	public int balls() {
		return balls;
	}
	
	/**
	 * Reset the statistics for a new ball.
	 *
	 */
	public void newBall() {
		startBallTime=System.currentTimeMillis();
		ballInPlay=true;
		ballLaunchable=false;
		GameOptions.getPlunger().reset();
		balls++;
		hits=0;	
	}

	/**
	 * Reset the statistics for a new game.
	 *
	 */	
	public void newGame() {
		startGameTime=System.currentTimeMillis();
		startButton.press();
	}
	
	/**
	 * Get the total time the ball was in play. 
	 *
	 * @return time in use.
	 */
	public long ballTime() {
		return ballTime;
	}
	
	/**
	 * Get the average time the ball was in play. 
	 *
	 * @return time in use.
	 */
	public float avgBallTime() {
		return (float)totalTime/balls;
	}

	/**
	 * Get the average number of hits. 
	 *
	 * @return average number of hits.
	 */
	public float avgHits() {
		return (float)hitsTotal/balls;
	}
	
	/**
	 * Ball is out of play. Calculate statistics. 
	 */	
	public void ballDone() {
		stoppedTime=System.currentTimeMillis();
		ballInPlay=false;
		lastSeen=0;
		ballLaunchable=true;
		ballTime=System.currentTimeMillis()-startBallTime;
		totalTime+=ballTime;
		hitsTotal+=hits;
		GameOptions.ballOutOfPlay(false);
		
		GameOptions.ballInPlay(balls);				
		
	}
	
	/**
	 * Check to see if the system is done doing the statistics for the game.
	 * System will exit when the testing is done.
	 */
	public void statsDone() {
	
		/* When the amount of balls in play have been played, exit. */
		if(balls==(GameOptions.ballsToPlay()))
			GameOptions.exitSystem();

	}	
	
	/**
	 * Check if the ball is about to be lanuched.
	 * 	
	 * @return Launchable status.
	 */
	public boolean ballLaunchable() {
		if(ballLaunchable && (System.currentTimeMillis()-stoppedTime)>launchTime) {
			stoppedTime=System.currentTimeMillis();
			return true;
		}
		return false;
	}
			
	
	/** 
	 * The ball was not found in the system.
	 */
	public void ballDisappeared() {
		if(lastSeen==0) {
			lastSeen=System.currentTimeMillis();
		}
		
		else if((System.currentTimeMillis()-lastSeen)>missingTime) {
			startBallTime-=missingTime;
			GameOptions.ballOutOfPlay(true);
			lastSeen=0;
		}
	}
	
	/**
	 * The ball is found in the system.
	 */
	public void ballAlive() {
		lastSeen=0;
	}
		
	
	
	/**
	 * Ball is in play.
	 */
	public boolean ballInPlay() {
		return ballInPlay;
	}
				
	/**
	 * Set "ball is in play" status.
	 *
	 * @param b call in play status.
	 */
	public void ballInPlay(boolean b) {
		ballInPlay=b;
	}
				
			
}
		
	
