package pinball.buffer;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.io.*;

/** 
 * The buffer. Has a information buffer containing data collected by the frame.
 * The buffers are stored as a circular linked list. 
 * @see pinball.buffer.BuffInfo
 */
public class Buffer {
	/** The lock for the buffer. */
	private Watcher lock;
  	
  	/** The next buffer. */
  	private transient Buffer next;
	
	/** The information of the buffer. */
	private BufferInfo info; 
	
	/** The buffer's id. */
	private int id;

	/** The strategy's unlock status. Determines whether the buffer was unlocked
	 * naturally or forced unlocked.
	 * Naturally means that it was unlocked properly.
	 * Forced means that it never got the signal to unlock and was forced unlocked
	 * by another buffer being unlocked. This is used in network play, should 	 
	 * an unlock command be lost in the network.
	 */
	private String strategyUnlock;
	
	/** Indicates if the buffer was skipped in by a thread. */
	private boolean skipped=false;

	/** Is this buffer able to be recorded. */
	private boolean recordable=false;


	/**
	 * Set's the id of the buffer.
	 *
	 * @param id the id for this buffer.
 	 */
	public Buffer(int id) {
		lock=new Watcher();
		next=null;
		info = new BufferInfo(id);
		this.id=id;
	}
	
	/**
	 * For extended use.
	 */
	public Buffer() {}

	/** 
	 * Was the buffer skipped by a thread.
	 *
	 * @return skipped status.
	 */
	public boolean skipped() {
		return skipped;
	}
	
	/** Set skip status.
	 * 
	 * @param s skip status.
	 */
	public void skipped(boolean s) {
		skipped=s;
	}

	/**
	 * Set's the id of the buffer.
	 *
	 * @param id the id for this buffer.
	 * @param info the information of the buffer.
	 * @param lock The lock for the buffer. Used in clone.
 	 */
	public Buffer(int id, BufferInfo info, Watcher lock) {
		this.lock=lock;
		next=null;
		this.info = info;
		this.id=id;
	}
	

	/**
	 * Used to clone the buffer.
	 *
	 * @return clone of buffers.
	 */	
	public Object clone() {
		return new Buffer(id,info,lock);
	}	
	
				
	
	/**
	 * Set the next buffer.
	 *
	 * @param buffer The next buffer.
	 */
	public void next(Buffer buffer) {
		next=buffer;
	}

	
	/** 
	 * Get the next buffer. 
	 *
	 * @param the next buffer.
	 */
	public Buffer next() {
		return next;
	}

	/**
	 * The mutex for the buffer in the processing threads.
	 *
	 * @return the lock for the buffer. 
	 */
	public Watcher lock() {
		if(GameOptions.debug()>=2)
			System.err.println("Frame ID: "+id);
		return lock;
	}

	/**
	 * Get the buffer info. 
	 *
	 * @return the information buffer.
	 */
	public BufferInfo info() {
		return info;
	}
	
	/** Set the information. 
	 * 
	 * @param i information buffer to be set.
	 */
	public void info (BufferInfo i) {
		info=i;
	}
	
	/** 
	 * Get the buffer's id.
	 *
	 * @return the id of this buffer.
	 */
	public int id() {
		return id;
	}
	
	/** 
	 * Get's the strategy unlock status.
	 *
	 * @return strategy unlock status. */
	public void strategyUnlock(String u) {
		strategyUnlock=u;
	}

	/** 
	 * The buffer can be recorded.
	 */
	public void recordable() {
		recordable=true;
	}
	
	/**
	 * Returns recordable status. 
	 *
	 * @return Status of buffer's recordablity.
	 */
	public boolean canRecord() {
		return recordable;
	}

}

