package pinball.io;

import java.io.*;
import java.util.*;

/**
 * Class for allowing IO via the serial port.
 */
public class SerialIO {	
	/** Test port value. */
	private final String testPort = "/dev/cuaa0";

	/** The serial port to write the data to. */
	private SerialPort serialPort;
	
	/** Data read from the port. */
	private Vector inputData=new Vector();

	/** Data to be written to the port. */
	private Vector outputData=new Vector();
	
	/** The last byte read in, so the system can store the pairs of bytes. */
	private byte lastData;
	
	/** Selects if the lastData byte is not to be stored. */
	private int byteToggle=0;
	
	/**
	 * Make the serial port with the port given.
	 * 
	 * param port serial port location.
	 */		
	public SerialIO(String port){	
		serialPort=new SerialPort(port,this);
		serialPort.start();	
	}
	
	/**
	 * Read from the port.
	 *
	 * return the byte read from the port.
	 */
	public synchronized byte read() throws IOException{
		byte b=0;
		
		serialPort.wakeup();
		
		Thread.yield();
		
		/* If there is data, get it. */
		if(inputData.size()>0) {
			b=((Byte)inputData.elementAt(0)).byteValue();
			inputData.removeElementAt(0);
		}			
		
		else throw new IOException("No data is available from the serial port");
		
//		System.err.println("READ "+b);
		
		return b;
	}		
	
				
	/**
	 * Clear any data waiting to be read. 
	 */
	public synchronized void flushInputData() {
		inputData.removeAllElements();
		
		/* Clear the last item read in. */
		lastData=-1;
	}

	/**
	 * Clear any commands waiting to go out. 
	 */
	public synchronized void flushOutputData() {
		outputData.removeAllElements();
	}

	/**
	 * Add data that read in from the serial port. 
	 *
	 * @param data the byte read from the port.
	 */	
	public synchronized void setReadData(byte data) {
		/* Setup the msb of the data. */
		if(byteToggle==0) {
			lastData=data;
			byteToggle=1;
		}
			
		/* Store the msb and the lsb of the data. */
		else if(byteToggle==1) {
			inputData.addElement(new Byte(lastData));
			inputData.addElement(new Byte(data));
			byteToggle=0;
		}
		
	}		
	
	
	/**
	 * Write the data to the port.
	 *
	 * param data the data to write. 
	 */
	public synchronized void write(String data) throws IOException{
		outputData.addElement(data);
		serialPort.wakeup();
		Thread.yield();
//		System.err.println("WRITE "+data);		
	}
		
	/**
	 * Checks if there is any available data to be written to the serial port.
	 *
	 * @return if output data is available.
	 */
	public synchronized boolean outputAvailable() {
		/* Check if there is data available. */
		if(outputData.size()>0) {
//			System.err.println("WAITING "+outputData.size());	
			return true;
		}
		else return false;
	}


	/**
	 * Gets the next command to be written to the serial port.
	 *
	 * @return The data to be written to the serial port.
	 */
	public synchronized Vector outputData() {
		Vector data=outputData;
		
		outputData=new Vector();
		
		return data;
	}
	
}
