package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * Decide whether to flip the flipper or not. 
 */
public class Strategy extends Thread {

	
	/** X location of the ball. */
	protected	int x;
	
	/** Y location of the ball. */
	protected 	int y;
	
	/** Image x size. */
	protected	int xsize; 
	
	/** Image y size. */
	protected	int ysize;
	
	/** Left flipper x location. */
	protected int leftX=0;
	
	/** Left flipper y location. */
	protected int leftY=0;
	
	/** Right flipper x location. */	
	protected int rightX=0;
	
	/** Right flipper y location. */	
	protected int rightY=0;
	
	/** Area around flipper that is to be ignored. */
	protected int boarder=1;
		
	/** Where is the flipper from the top of the screen. */
	protected int flipperOffset=0;
	
		
	/** How long can the flipper stay up in the game. */
	protected int resetTime=GameOptions.resetTime();		

	/** Y location of begin flipper zone. */
	protected int flipperZone;
	
	/** Used in options to see if the they happened once. */
	protected boolean firstTime=true;
	
	/** Location from top of screen to be considered in play. */
	protected int fromTop=15;
	
	/** Location from bottom of screen to be considered out of play. */
	protected int fromBottom=3;
	
	/** Anything moving below this be considered for flipping. */
	protected int flipperHeight=0;
	
	/** 
	 * Location from bottom of screen to be considered out of play. 
	 * Gets value from fromTop.
	 */
	protected int top=fromTop;
	
	/** 
	 * Location from bottom of screen to be considered out of play.
	 * Gets value from fromBottom. 
	 */
	protected int outZone=0;

	
	/** The information buffer. */
	protected Buffer buffer;


	/** The information from the buffer. */
	protected BufferInfo buffinfo;
	
	/** ID of strategy. */
	protected String id="Standard";


	/**
	 * The stub to connect the strategy to the system.
	 */
	protected Stub stub;
	

	/** The lock on the strategy. */
	protected boolean key=false;

	/** The queue of buffers waiting to be looked at. */
	protected StrategyQueue queue;
	
	/** The distance the ball is from the left flipper. */
	protected int leftDistance=0;

	/** The distance the ball is from the right flipper. */	
	protected int rightDistance=0;
	
	/** The x location for the middle of the flippers. */
	protected int middle=0;
	
	
	/**
	 * Make a new strategy thread. 
	 *
	 * @param buffer the first buffer in the series of buffer.
	 * @param gameStats the area to write the game stats info.
	 * @param camera the camera for the system.	 
	 */
	public Strategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.stub=new Stub(buffer,gameStats,this,camera);

		boarder=1;				
		stub.setPriority(Thread.NORM_PRIORITY);
		stub.boarder(boarder);
		System.err.println("Strategy: "+id);
		GameOptions.ballMax(6);
		GameOptions.moveThreshold(5);
		
		System.err.println("***STANDARD STRATEGY***");

	}
	
	/**
	 * Used in the extended classed. 
	 */
	public Strategy() {}
	
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "Strategy";
	}


	/**
	 * Get the id of the strategy.
	 *
	 * @return id of strategy. 
	 */
	public String id() {
		return id;
	}
		
		
	/** Allow extended classes to define a custom startup option. */
	public void custom(){}
	
	/** Wait until the stub filtered out the ball and then decide to flip or not. */
	public void run() {
		stub.start();
		
		custom();
		
		while(true) {
			halt();			

			buffer=stub.getRecentBuffer();			
			if(buffer!=null) {
				buffinfo=buffer.info();
				option();
				getStubInfo();	
				flippersCheck();
				
				stub.releaseLock(buffer);
			}
			
			if(buffer==null) 
				System.err.println("Why no buffer?");

			Thread.yield();
		}
	}
	
	/**
	 * Halt the thread until the queue is not empty. 
	 */	
	public synchronized void halt() {
		while(stub.isQueueEmpty()) {
			try {
	                        wait();
			} 
			catch (InterruptedException e) {System.err.println(e);}
		}
	}
	
	/**
	 * Notify the thread that a buffer is available. 
	 */
	public synchronized void bufferAvailable() {
		notifyAll();
	}
		
	/**
	 * Set up the queue of buffers.
	 *
	 * @param q the queue of buffers.
	 */
	public void setQueue(StrategyQueue q) {
		queue=q;
	}
	
	/**
	 * Used to simulate lots packets. This will randomly release the lock.
	 */
	public boolean testing() {
		double n=java.lang.Math.random();
		
		if(n>=.6)
			return true;
		return false;
	}

	
	/**
	 * Initalize the strategy. Finds the flippers.
	 */
	public void initalize() {		
		
	}
	
	/**
	 * Get the stub from the strategy.
	 */
	public Stub getStub() {
		return stub;
	}
	
	/**
	 * Get info from the stub. The flipper locations, and flipper zone info.
	 */
	public void getStubInfo() {
		leftX=stub.leftX();
		leftY=stub.leftY();
		rightX=stub.rightX();
		rightY=stub.rightY();
		outZone=stub.outZone();
		flipperZone=stub.flipperZone();
		middle=(leftX+rightX)/2;
	}
	
	/** 
	 * Get the flipper reset time.
	 *
	 * @param reset the reset time.
	 */
	public void resetTime(int reset) {
		resetTime=reset;
	}
	
	/** 
	 * Reset the flippers if they are up the allowed amount of time.
	 */ 
	public void flippersReset() {		
		if(stub.leftFlipperReset(resetTime) && stub.isLeftUp()) {
			stub.leftFlipperState(false);
		}
		if(stub.rightFlipperReset(resetTime) && stub.isRightUp()) {
			stub.rightFlipperState(false);
		}
	}

	
	/**
	 * Set the flippers flipped condition. 
	 *
	 * @param status the flippers flipped status.
	 */
	public void flipperFlipped(boolean status) {
		stub.flipperFlipped(status);
	}
	
	
	/**
	 * Check if the flippers need to be reset and then decide if they need to be flipped.
	 */
	public void flippersCheck() {
		
		/* Descide whether to flip. */
		if(stub.statsBallInPlay())
			decision();
	
		/* See if it's time to reset the flippers. */	
		flippersReset();


	} 
	 
	
	/**
	 * Setup the options for the strategy. 
	 * This one sets the in play area and flipper height.
	 */
	public void option() {				
		flipperHeight=stub.height()/2;
		top=fromTop;	
		
		stub.topBottom(top,outZone);
		
		
		if(firstTime) {
			stub.headerInfo("flipperHeight="+flipperHeight);
			stub.headerInfo("top="+top);
			stub.headerInfo("outZone="+outZone);
			stub.headerInfo("flipperDistance="+boarder);
						
			firstTime=false;
		}
	}
			
	/** 
	 * Should you flip or not. 
	 */
	public void decision() {		

		/* Set the default that it did not flip. */	
		flipperFlipped(false);
		
		/* Is there a ball to flip? If so flip it.*/
		/* Add the offset value for the ball's location so it's
		correctly found from original location in game. */
		
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
		
//		System.err.println("Ball: "+x+","+y+" Flipper height "+flipperHeight);
//		System.err.println("LeftX: "+leftX+" RightX: "+rightX);
			
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
			
		/* Flip the flippers. */
		//if(x>=0 && y>=flipperHeight) {
		if(x>=0) {
		
			/* Check for valid movement. Make sure it's not the flipper that
			is seens as movement.*/
			
				/* Flip the flipper to the corresponding half of the frame
				where the ball is found. */
			
				/* Find the distance of the ball from the flippers, 
				   to see which to flip. 
				*/
				if(x!=-1) {
					leftDistance=leftX-x;
					if(leftDistance<0)
						leftDistance=-leftDistance;

					rightDistance=rightX-x;
					if(rightDistance<0)
						rightDistance=-rightDistance;
					
				}
			
				/* Flip left flipper. */
				if(leftDistance<=rightDistance && stub.isLeftDown()) {
					stub.leftFlipperState(true);
				 }
				/* Flip right flipper. */
				if(rightDistance<=leftDistance && stub.isRightDown()) {
					stub.rightFlipperState(true);
				 }
		}

	}
	
	/**
	 * Allows the strategy to draw to the screen any lines or other graphics
	 * for the user to see how the strategy operates.
	 *
	 * @param g the graphics area to draw to.
	 * @param canvasWidth the width of the canvas.
	 * @param canvasHeight the height of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
	
	}
	
}
